import pytest

@pytest.fixture
def test_ip():
    return "192.168.1.5"

@pytest.fixture
def url(test_ip):
    return f"http://{test_ip}/data.json"

@pytest.fixture
def json_data():
    return {
        "software_version": "R_2025-03",
        "sensor_id": "5366960e8b18",
        "sensordatavalues": [
            {"value_type": "signal", "value": "-48"},
            {"value_type": "SDS_P1", "value": "1.55"},
            {"value_type": "SDS_P2", "value": "0.508333"},
            {"value_type": "BME280_humidity", "value": "40.03418"},
            {"value_type": "BME280_temperature", "value": "21.34277"},
            {"value_type": "BME280_pressure", "value": "101636.6"},
            {"value_type": "PCBA_noiseMax", "value": "59"},
            {"value_type": "PCBA_noiseAvg", "value": "53"},
        ],
    }



"""Module for discovering Altruist sensors using zeroconf."""

import asyncio
import logging

from zeroconf import ServiceStateChange, Zeroconf
from aiohttp import ClientError, ClientSession
from zeroconf.asyncio import AsyncServiceBrowser, AsyncServiceInfo, AsyncZeroconf

from .client import AltruistClient
from .model import AltruistDeviceModel
from .errors import AltruistError

_LOGGER: logging.Logger = logging.getLogger(__name__)

_PENDING_TASKS: set[asyncio.Task] = set()


class AltruistDiscoverer:
    """Discover ESP32 sensors using zeroconf."""

    def __init__(self, zeroconf_instance: AsyncZeroconf, session: ClientSession) -> None:
        """Initialize the discoverer.

        :param zeroconf_instance: An instance of AsyncZeroconf.
        """
        self.zeroconf: Zeroconf = zeroconf_instance.zeroconf
        self.session = session
        self._devices: list[AltruistDeviceModel] = []

    async def get_devices(self) -> list[AltruistDeviceModel]:
        """Discover devices advertising the given service type.

        This method waits for a fixed interval to collect available devices.
        If no device is discovered, it raises DevicesNotFoundException.

        :return: A list of ServiceInfo objects for the discovered devices.
        :raises DevicesNotFoundException: If no devices are found.
        """
        # Clear previous results.
        self._devices.clear()
        services = ["_altruist._tcp.local."]
        aiobrowser = AsyncServiceBrowser(
            self.zeroconf, services, handlers=[self._async_on_service_state_change]
        )
        await asyncio.sleep(5)

        return self._devices

    def _async_on_service_state_change(
        self, zeroconf: Zeroconf, service_type: str, name: str, state_change: ServiceStateChange
    ) -> None:
        _LOGGER.debug(f"Service {name} of type {service_type} state changed: {state_change}")
        if state_change is not ServiceStateChange.Added:
            return
        task = asyncio.ensure_future(self._async_add_device(zeroconf, service_type, name))
        _PENDING_TASKS.add(task)
        task.add_done_callback(_PENDING_TASKS.discard)


    async def _async_add_device(self, zeroconf: Zeroconf, service_type: str, name: str) -> None:
        info = AsyncServiceInfo(service_type, name)
        await info.async_request(zeroconf, 3000)
        ip = info.parsed_addresses()[0]
        try:
            client = await AltruistClient.from_ip_address(self.session, ip)
            device = client.device
            self._devices.append(device)
            _LOGGER.debug(f"Added device: {device}")
        except AltruistError:
            _LOGGER.warning("Skipping unreachable or malformed device at %s", ip)




"""Module for Altruist device model."""

from dataclasses import dataclass

from zeroconf.asyncio import AsyncServiceInfo


@dataclass
class AltruistDeviceModel:
    """Data class for storing information about an Altruist device."""

    id: str
    ip_address: str
    name: str = "Altruist Sensor"
    fw_version: str | None = None



import pytest
from aiohttp import ClientSession
from aioresponses import aioresponses
from altruistclient.client import AltruistClient
from altruistclient.model import AltruistDeviceModel


@pytest.mark.asyncio
async def test_from_ip(test_ip, url, json_data):
    device = AltruistDeviceModel(id="5366960e8b18", ip_address=test_ip)
    async with ClientSession() as session:
        with aioresponses() as mocked:
            mocked.get(url, payload=json_data)
            client = await AltruistClient.from_ip_address(session, test_ip)
            assert client.device.id == "5366960e8b18"
            assert client.device.ip_address == test_ip
            assert client.fw_version == "R_2025-03"


@pytest.mark.asyncio
async def test_fetch_data(test_ip, url, json_data):
    device = AltruistDeviceModel(id="5366960e8b18", ip_address=test_ip)
    async with ClientSession() as session:
        with aioresponses() as mocked:
            mocked.get(url, payload=json_data)
            client = AltruistClient(session, device)
            data = await client.fetch_data()
            assert "SDS_P1" in client.sensor_names
            assert "SDS_P2" in client.sensor_names
            assert "BME280_humidity" in client.sensor_names
            assert "BME280_temperature" in client.sensor_names
            assert "BME280_pressure" in client.sensor_names
            assert "PCBA_noiseMax" in client.sensor_names
            assert "PCBA_noiseAvg" in client.sensor_names
            assert "signal" in client.sensor_names
            assert client.fw_version == "R_2025-03"
            assert len(data) == 8



"""Module for initialization the Altruist sensor component."""

from .client import AltruistClient
from .discoverer import AltruistDiscoverer
from .model import AltruistDeviceModel
from .errors import AltruistError

__all__ = ["AltruistClient", "AltruistDeviceModel", "AltruistDiscoverer", "AltruistError"]


from copy import deepcopy
import json
import logging

from aiohttp import ClientError, ClientSession
from aiohttp.client_exceptions import ClientConnectorError

from .model import AltruistDeviceModel
from .errors import AltruistError

_LOGGER: logging.Logger = logging.getLogger(__name__)


class AltruistClient:
    """Class for handling the data retrieval."""

    def __init__(self, session: ClientSession, device: AltruistDeviceModel) -> None:
        """Initialize the data object."""
        self._session = session
        self._resource = f"http://{device.ip_address}/data.json"
        self.device = device
        self._last_data: list | None = None

    @classmethod
    async def from_ip_address(
        cls, session: ClientSession, ip_address: str
    ) -> "AltruistClient":
        """Create client from given IP address."""
        try:
            response = await session.get(f"http://{ip_address}/data.json")
        except ClientConnectorError:
            _LOGGER.error("Can't connect to Altruist Sensor on %s", ip_address)
            raise AltruistError
        responseData = await response.text()
        _LOGGER.debug("Received data: %s", str(responseData))
        parsed_json = json.loads(responseData)
        if "software_version" in parsed_json:
            device = AltruistDeviceModel(
                id=parsed_json.get("sensor_id", ip_address),
                ip_address=ip_address,
                fw_version=parsed_json["software_version"],
            )
            return cls(session, device)
        raise AltruistError

    @property
    def fw_version(self) -> str | None:
        """The firmware version of the sensor."""
        return self.device.fw_version

    @property
    def device_id(self) -> str:
        """The mac address of the sensor."""
        return self.device.id

    @property
    def sensor_names(self) -> list[str]:
        """Get the list of the sensor measurements names."""
        if self._last_data:
            return [
                item["value_type"]
                for item in self._last_data
                if self._valid_sensor_name(item["value_type"])
            ]
        return []

    def _valid_sensor_name(self, sensor_name: str) -> bool:
        """Return True if the sensor name is supported."""
        if "GPS" in sensor_name:
            return False
        if "samples" in sensor_name:
            return False
        if "micro" in sensor_name:
            return False
        if "interval" in sensor_name:
            return False
        return True

    async def fetch_data(self) -> dict:
        """Get the latest data from Altruist device."""
        responseData = None
        try:
            _LOGGER.debug("Get data from %s", str(self._resource))
            response = await self._session.get(self._resource)
            responseData = await response.text()
            _LOGGER.debug("Received data: %s", str(responseData))
        except ClientError as err:
            _LOGGER.warning("REST request error: %s", err)
            raise AltruistError from err
        except TimeoutError:
            _LOGGER.warning("REST request timeout")
            raise AltruistError from TimeoutError

        try:
            parsed_json = json.loads(responseData)
            if not isinstance(parsed_json, dict):
                _LOGGER.warning("JSON result was not a dictionary")
                parsed_json = {}
            self.device.fw_version = parsed_json.get("software_version")
            data = parsed_json.get("sensordatavalues", {})
        except ValueError:
            _LOGGER.warning("REST result could not be parsed as JSON")
            raise AltruistError from ValueError
        self._last_data = deepcopy(data)
        return data
